class YamlCodes:
    """
    These are the standardized codes mapped in the YAML files that
    detect variants.
    """
    DATE = 'date'
    DATE_TIME = "datetime"
    EASTING = "easting"
    EPSG = 'epsg'
    ID_NAME = "pit_id"
    LATITUDE = "latitude"
    LONGITUDE = "longitude"
    NORTHING = "northing"
    SITE_NAME = 'site_name'
    TIME = 'time'
    UTM_EPSG_PREFIX = "269"
    UTM_ZONE = 'utm_zone'
